Class: Picky::Generators::Partial::None
- Defined in:
- lib/picky/generators/partial/none.rb
Overview
Does not generate a partial index.
Instance Method Summary collapse
-
#each_partial(token) ⇒ Object
Yields each generated partial.
-
#saved? ⇒ Boolean
Returns if this strategy’s generated file is saved.
-
#use_exact_for_partial? ⇒ Boolean
Do not use the partial bundle for getting ids and weights.
Methods inherited from Strategy
Instance Method Details
#each_partial(token) ⇒ Object
Yields each generated partial.
13 14 15 |
# File 'lib/picky/generators/partial/none.rb', line 13 def each_partial token # yields nothing end |
#saved? ⇒ Boolean
Returns if this strategy’s generated file is saved.
19 20 21 |
# File 'lib/picky/generators/partial/none.rb', line 19 def saved? false end |
#use_exact_for_partial? ⇒ Boolean
Do not use the partial bundle for getting ids and weights.
25 26 27 |
# File 'lib/picky/generators/partial/none.rb', line 25 def use_exact_for_partial? true end |