Class: Picky::Generators::Partial::Postfix
- Defined in:
- lib/picky/generators/partial/postfix.rb
Instance Method Summary collapse
-
#initialize(options = {}) ⇒ Postfix
constructor
The from option signifies where in the symbol it will start in generating the subtokens.
Methods inherited from Substring
Methods inherited from Strategy
Methods inherited from Strategy
Constructor Details
#initialize(options = {}) ⇒ Postfix
The from option signifies where in the symbol it will start in generating the subtokens.
Examples:
With :hello
-
from: 1 # => [:hello, :hell, :hel, :he, :h]
-
from: 4 # => [:hello, :hell]
18 19 20 21 22 |
# File 'lib/picky/generators/partial/postfix.rb', line 18 def initialize = {} [:to] = -1 super end |