Class: Picky::Wrappers::Bundle::ExactPartial

Inherits:
Wrapper show all
Defined in:
lib/picky/wrappers/bundle/exact_partial.rb

Overview

This index combines a partial and exact bundle such that a partial index will not be dumped or generated.

Instance Attribute Summary

Attributes inherited from Wrapper

#bundle

Instance Method Summary collapse

Methods inherited from Wrapper

#initialize

Constructor Details

This class inherits a constructor from Picky::Wrappers::Bundle::Wrapper

Instance Method Details

#clearObject

Ignore these.



17
# File 'lib/picky/wrappers/bundle/exact_partial.rb', line 17

def clear; end

#dumpObject



18
# File 'lib/picky/wrappers/bundle/exact_partial.rb', line 18

def dump; end

#emptyObject



19
# File 'lib/picky/wrappers/bundle/exact_partial.rb', line 19

def empty; end

#indexObject



20
# File 'lib/picky/wrappers/bundle/exact_partial.rb', line 20

def index; end

#loadObject



21
# File 'lib/picky/wrappers/bundle/exact_partial.rb', line 21

def load(*); end

#to_tree_s(indent = 0) ⇒ Object



23
24
25
# File 'lib/picky/wrappers/bundle/exact_partial.rb', line 23

def to_tree_s indent = 0
  bundle.to_tree_s(indent) { "(reference)" }
end