Class: Picky::Generators::Partial::Strategy
- Defined in:
- lib/picky/generators/partial/strategy.rb
Overview
Superclass for partial strategies.
Direct Known Subclasses
Instance Method Summary collapse
-
#use_exact_for_partial? ⇒ Boolean
Defines whether to use the exact bundle instead of the partial one.
Methods inherited from Strategy
Instance Method Details
#use_exact_for_partial? ⇒ Boolean
Defines whether to use the exact bundle instead of the partial one.
Default is @false@.
For example:
Partial::None.new # Uses the exact index instead of the partial one.
19 20 21 |
# File 'lib/picky/generators/partial/strategy.rb', line 19 def use_exact_for_partial? false end |