Class: Picky::Generators::Similarity::DoubleMetaphone
- Inherits:
-
Phonetic
- Object
- Picky::Generators::Strategy
- Phonetic
- Picky::Generators::Similarity::DoubleMetaphone
- Defined in:
- lib/picky/generators/similarity/double_metaphone.rb
Overview
It’s actually a combination of double metaphone and Levenshtein.
It uses the double metaphone to get similar words and ranks them using the levenshtein.
Instance Attribute Summary
Attributes inherited from Phonetic
Instance Method Summary collapse
-
#encode(str_or_sym) ⇒ Object
Encodes the given string/symbol.
Methods inherited from Phonetic
#check_gem, #initialize, #prioritize
Methods inherited from Picky::Generators::Strategy
Constructor Details
This class inherits a constructor from Picky::Generators::Similarity::Phonetic
Instance Method Details
#encode(str_or_sym) ⇒ Object
Encodes the given string/symbol.
Returns a symbol.
21 22 23 |
# File 'lib/picky/generators/similarity/double_metaphone.rb', line 21 def encode str_or_sym str_or_sym. end |