Class: Picky::Interfaces::LiveParameters::Unicorn
- Inherits:
-
MasterChild
- Object
- MasterChild
- Picky::Interfaces::LiveParameters::Unicorn
- Defined in:
- lib/picky/interfaces/live_parameters/unicorn.rb
Overview
This is an interface that provides the user of Picky with the possibility to change parameters while the Application is running.
Instance Method Summary collapse
Methods inherited from MasterChild
#close_child, #extract_configuration, #harakiri, #initialize, #kill_each_worker_except, #kill_worker, #parameters, #querying_removes_characters, #querying_removes_characters=, #querying_splits_text_on, #querying_splits_text_on=, #querying_stopwords, #querying_stopwords=, #start_master_process_thread, #to_s, #try_updating_configuration_with, #write_parent
Constructor Details
This class inherits a constructor from Picky::Interfaces::LiveParameters::MasterChild
Instance Method Details
#remove_worker(wpid) ⇒ Object
20 21 22 |
# File 'lib/picky/interfaces/live_parameters/unicorn.rb', line 20 def remove_worker wpid worker = Unicorn::HttpServer::WORKERS.delete(wpid) and worker.tmp.close rescue nil end |
#worker_pids ⇒ Object
16 17 18 |
# File 'lib/picky/interfaces/live_parameters/unicorn.rb', line 16 def worker_pids Unicorn::HttpServer::WORKERS.keys end |